package yuehaoxitong

import (
	"shylinux.com/x/ice"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/community/src/yuehaoxitong/model"
)

type schedule struct {
	Table
	queue     queue
	userQueue userQueue
	volume    volume
	reception reception
	portal    string `data:"true"`
	plan      string `name:"plan begin_time@date end_time@date"`
}

func (s schedule) Plan(m *ice.Message, arg ...string) {
	arg = kit.TransArgValueTime(arg, model.BEGIN_TIME, model.END_TIME)
	s.Table.Create(m, append(arg, m.OptionSimple(model.QUEUE_UID, model.USER_UID)...)...)
}
func (s schedule) Call(m *ice.Message, arg ...string) {
	s.Orders(m, model.TAKE_TIME).Limit(m, 1)
	s.SelectByStatus(m, m.Option(model.QUEUE_UID), kit.Format(ScheduleTake))
	// s.Select(m, kit.Simple(m.OptionSimple(model.QUEUE_UID, model.RECEPTION_UID), mdb.STATUS, ScheduleTake)...)
	if m.WarnNotFound(m.Length() == 0) {
		return
	}
	s.ChangeStatus(m, m.Append(model.UID), "", ScheduleTake, ScheduleCall, arg...)
}
func (s schedule) SelectByStatus(m *ice.Message, arg ...string) {
	s.Tables(m, s.volume).FieldsWithCreatedAT(m, s,
		model.SCHEDULE_STATUS, s.Key(s.volume, model.BEGIN_TIME), s.Key(s.volume, model.END_TIME), s.Key(s, model.RECEPTION_UID), model.USER_UID,
	).Orders(m, s.Desc(model.BEGIN_TIME))
	s.Select(m, kit.Simple(s.Key(s, model.QUEUE_UID), arg[0], model.STATUS, arg[1], arg[2:])...)
	s.RewriteAppend(m)
	s.SelectJoin(m, s.reception)
	s.SelectJoinUser(m)
}
func (s schedule) List(m *ice.Message, arg ...string) {
	if s.CheckRole(m.Options(model.QUEUE_UID, arg[0]), UserQueueCreator, UserQueueServer) {
		if len(arg) == 1 {
			s.Select(m, model.QUEUE_UID, arg[0])
		} else if len(arg) == 2 {
			s.SelectDetail(m, model.QUEUE_UID, arg[0], model.UID, arg[1])
		}
	} else {
		if len(arg) == 1 {
			s.Select(m, model.QUEUE_UID, arg[0], model.USER_UID, m.Option(model.USER_UID))
		} else if len(arg) == 2 {
			s.SelectDetail(m, model.QUEUE_UID, arg[0], model.USER_UID, m.Option(model.USER_UID), model.UID, arg[1])
		}
	}
	m.RenameAppend(model.STATUS, model.SCHEDULE_STATUS)
	s.RewriteAppend(m)
	s.SelectJoin(m, s.reception)
	s.SelectJoinUser(m)
	kit.If(m.Length() == 0, func() { m.Echo(m.Trans("not found", "没有预约")) })
	m.PushAction().Action()
}
func (s schedule) Take(m *ice.Message, arg ...string) {
	s.ChangeStatus(m, m.Option(model.UID), "", SchedulePlan, ScheduleTake, arg...)
}
func (s schedule) Cancel(m *ice.Message, arg ...string) {
	s.ChangeStatus(m, m.Option(model.UID), "", SchedulePlan, ScheduleTake, arg...)
}
func (s schedule) Expire(m *ice.Message, arg ...string) {
	s.ChangeStatus(m, m.Option(model.UID), "", ScheduleCall, ScheduleExpire, arg...)
}
func (s schedule) Finish(m *ice.Message, arg ...string) {
	s.ChangeStatus(m, m.Option(model.UID), "", ScheduleCall, ScheduleFinish, arg...)
}

func init() { ice.TeamCtxCmd(schedule{}) }

func (s schedule) ChangeStatus(m *ice.Message, uid, key string, from, to ScheduleStatus, arg ...string) *ice.Message {
	kit.If(key == "", func() { key = m.ActionKey() + "_time" })
	s.Table.ChangeStatus(m, uid, int(from), int(to), kit.TransArgValueTime(append(arg, key, m.Time()), key)...)
	return m
}

type ScheduleStatus int

const (
	SchedulePlan ScheduleStatus = iota
	ScheduleCancel
	ScheduleTake
	ScheduleCall
	ScheduleExpire
	ScheduleFinish
)

var ScheduleStatusList = map[ScheduleStatus]string{
	SchedulePlan:   "plan",
	ScheduleCancel: "cancel",
	ScheduleTake:   "take",
	ScheduleCall:   "call",
	ScheduleExpire: "expire",
	ScheduleFinish: "finish",
}

func (s ScheduleStatus) String() string { return ScheduleStatusList[s] }
